import gleam/dynamic/decode.{type Decoder}
import gleam/json.{type Json}

pub type ReactorSymbol {
  ReactorSolarI
  ReactorFusionI
  ReactorFissionI
  ReactorChemicalI
  ReactorAntimatterI
}

pub fn parse(value: String) -> Result(ReactorSymbol, Nil) {
  case value {
    "REACTOR_SOLAR_I" -> Ok(ReactorSolarI)
    "REACTOR_FUSION_I" -> Ok(ReactorFusionI)
    "REACTOR_FISSION_I" -> Ok(ReactorFissionI)
    "REACTOR_CHEMICAL_I" -> Ok(ReactorChemicalI)
    "REACTOR_ANTIMATTER_I" -> Ok(ReactorAntimatterI)
    _ -> Error(Nil)
  }
}

pub fn decoder() -> Decoder(ReactorSymbol) {
  use value <- decode.then(decode.string)
  case parse(value) {
    Ok(reactor_symbol) -> decode.success(reactor_symbol)
    Error(Nil) -> decode.failure(ReactorSolarI, "ReactorSymbol")
  }
}

pub fn to_string(reactor_symbol: ReactorSymbol) -> String {
  case reactor_symbol {
    ReactorSolarI -> "REACTOR_SOLAR_I"
    ReactorFusionI -> "REACTOR_FUSION_I"
    ReactorFissionI -> "REACTOR_FISSION_I"
    ReactorChemicalI -> "REACTOR_CHEMICAL_I"
    ReactorAntimatterI -> "REACTOR_ANTIMATTER_I"
  }
}

pub fn encode(reactor_symbol: ReactorSymbol) -> Json {
  json.string(to_string(reactor_symbol))
}
